﻿default dungeon_zoom = 3
default dungeon_padding = 8

init python:
    from copy import copy
    class dungeon_card:
        def __init__(self, image, name, health = 0, damage = 0, type = "weapon"):
            self.image = image
            self.name = name
            self.visible = True
            self.parent = None
            self.health = health
            self.damage = damage
            self.type = type
    class dungeon_player(renpy.Displayable):
        def __init__(self, image, health = 13, damage = 2):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.x_index = 1
            self.y_index = 1

            self.width = (64 * dungeon_zoom) + dungeon_padding * 2
            self.height = (96 * dungeon_zoom) + dungeon_padding * 2
            self.x = (self.x_index * self.width) + dungeon_padding
            self.y = (self.y_index * self.height) + dungeon_padding
            self.x_target = self.x
            self.y_target = self.y

            self.rotation = 0
            self.rotation_direction = 1

            self.visible = True
            self.parent = None
            self.health = health
            self.damage = damage
        def move(self, y, x):
            if not x is None:
                self.x_index = x
                self.x_target = (x * self.width) + dungeon_padding
            if not y is None:
                self.y_index = y
                self.y_target = (y * self.height) + dungeon_padding
        def attack(self, y, x):
            if not x is None:
                x = (self.x_index + x)/2
                self.x_target = (x * self.width) + dungeon_padding
            if not y is None:
                y = (self.y_index + y)/2
                self.y_target = (y * self.height) + dungeon_padding
        def settle(self):
            self.x_target = (self.x_index * self.width) + dungeon_padding
            self.y_target = (self.y_index * self.height) + dungeon_padding

        def render(self,width,height,st,at):
            if self.visible:
                img = Composite(
                    (64*dungeon_zoom,96*dungeon_zoom),
                    (0,0), "card_base",
                    (16*dungeon_zoom,16*dungeon_zoom), self.image,
                    (0,0), Text(str(self.health))
                )
                walking = False
                if self.x < self.x_target:
                    self.x += 20
                    walking = True
                elif self.x > self.x_target:
                    self.x += -20
                    walking = True
                if self.y < self.y_target:
                    self.y += 20
                    walking = True
                elif self.y > self.y_target:
                    self.y += -20
                    walking = True
                if distance(self.x, self.x_target) < 20:
                    self.x = self.x_target
                if distance(self.y, self.y_target) < 20:
                    self.y = self.y_target
                if walking:
                    self.rotation += self.rotation_direction
                    if self.rotation > 6:
                        self.rotation_direction = -2
                    elif self.rotation < -6:
                        self.rotation_direction = 2
                else:
                    self.rotation = 0
                    self.settle()
                t = Transform(img, rotate = self.rotation, rotate_pad = False)
                child_render = renpy.render(t, width, height, st, at)
                cw, ch = child_render.get_size()

                rv = renpy.Render(cw, ch)
                rv.blit(child_render, (self.x, self.y))

                renpy.redraw(self, 0)
                return rv
            else:
                return renpy.Render(0,0)

    class dungeon_class:
        def __init__(self, cards):
            self.cards = cards
            self.field = []
            self.player = dungeon_player("card_player")
            self.generate()
        def reset(self):
            self.generate()
            self.player.move(1,1)
        def add_column(self):
            for row in self.field:
                card = copy(random.choice(self.cards))
                row.append(card)
        def clicked(self, target, yi, xi):
            if distance(self.player.x_index, xi) > 1 or distance(self.player.y_index, yi) > 1:
                return
            x = xi
            y = yi
            if self.player.x_index == xi:
                x = None
            if self.player.y_index == yi:
                y = None
            if x != None and y != None:
                return

            if target is None or target.type in ["food", "weapon", "item"]:
                self.player.move(y, x)
                return

            if target.type == "enemy":
                target.health -= self.player.damage
                self.player.attack(y, x)
                if target.health < 1:
                    self.field[yi][xi] = None 
        def generate(self):
            field = []
            for j in range(3):
                row = []
                for i in range(4):
                    card = copy(random.choice(self.cards))
                    row.append(card)
                field.append(row)
            field[1][1] = None
            self.field = field
init:
    image card_base:
        Crop((0,0,64,96), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_player:
        Crop((64,0,32,64), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_hamzilla:
        Crop((96,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_kithulu:
        Crop((128,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_demon_stump:
        Crop((160,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_bite_stool:
        Crop((192,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_chest_closed:
        Crop((224,0,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_fat_chick:
        Crop((96,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_punch_rat:
        Crop((128,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_eye_fly:
        Crop((160,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_mimic:
        Crop((192,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_open_chest:
        Crop((224,32,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_cake:
        Crop((64,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_meat:
        Crop((96,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_potion:
        Crop((128,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_elixir:
        Crop((160,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_campfire:
        Crop((192,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_backpack:
        Crop((224,64,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True

    image card_knife:
        Crop((0,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_axe:
        Crop((32,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_hammer:
        Crop((64,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_staff:
        Crop((96,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_cutlass:
        Crop((128,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_sword:
        Crop((160,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_horseshoe:
        Crop((192,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
    image card_gold:
        Crop((224,96,32,32), "card dungeon/sprites.png")
        zoom dungeon_zoom nearest True
transform card_dungeon_highlight:
    on idle:
        ease .2 additive .0
    on hover:
        ease .2 additive .2
style card_dungeon_text:
    font "card dungeon/tall pixels 2x.ttf"
    size 24
screen card_dungeon_screen():
    default g = dungeon_class(
        [
            dungeon_card("card_hamzilla", "HamZilla", 1, 1, "enemy"),
            dungeon_card("card_kithulu", "Kithullu", 1, 2, "enemy"),
            dungeon_card("card_bite_stool", "Bite stool", 2, 1, "enemy"),
            dungeon_card("card_demon_stump", "Demon stump", 5, 1, "enemy"),
            dungeon_card("card_fat_chick", "Fat chick", 2, 3, "enemy"),
            dungeon_card("card_punch_rat", "Punch rat", 3, 3, "enemy"),
            dungeon_card("card_eye_fly", "Eye fly", 2, 4, "enemy"),
            dungeon_card("card_mimic", "Mimic", 4, 3, "enemy"),
            dungeon_card("card_chest_closed", "Chest", 0, 0, "chest"),
            dungeon_card("card_cake", "Cake", 1, 2, "food"),
            dungeon_card("card_meat", "Meat", 3, 3, "food"),
            dungeon_card("card_potion", "Potion", 5, 0, "food"),
            dungeon_card("card_elixir", "Elixir", 5, 0, "elixir"),
            dungeon_card("card_campfire", "Campfire", 0, 0, "rest"),
            dungeon_card("card_backpack", "Backpack", 0, 0, "rest"),
            dungeon_card("card_knife", "Knife", 0, 1),
            dungeon_card("card_axe", "Axe", 0, 2),
            dungeon_card("card_hammer", "Hammer", 0, 3),
            dungeon_card("card_staff", "Staff", 0, 4),
            dungeon_card("card_cutlass", "Cutlass",0 ,5),
            dungeon_card("card_sword", "Sword", 0, 6),
            dungeon_card("card_horseshoe", "Horseshoe", 0, 0, "item"),
            dungeon_card("card_gold", "Gold", 0, 0, "item"),
        ]
    )
    style_prefix "card_dungeon"
    fixed fit_first True:
        align .5,.5
        vbox:
            spacing 0
            for y,j in enumerate(g.field):
                hbox:
                    spacing 0
                    for x,i in enumerate(j):
                        button:
                            padding dungeon_padding,dungeon_padding
                            action Function(g.clicked, i, y, x)
                            at card_dungeon_highlight
                            fixed fit_first True:
                                add "card_base"
                                if i:
                                    add i.image align .5,1.0 yoffset -16*dungeon_zoom
                                    fixed:
                                        xysize 39,45 align .0,.0
                                        text str(i.health) align .5,.5 color "#ac3232"
                                    fixed:
                                        xysize 39,45 align 1.0,.0
                                        text str(i.damage) align .5,.5 color "#9badb7"
                                    text str(i.name) align .5,1.0 color "#9badb7" yoffset -10
        add g.player align .0,.0

    vbox:
        button:
            text "Reset"
            action Function(g.reset)
        button:
            text "Add column"
            action Function(g.add_column)
        button:
            text "Exit"
            action Return()


label card_dungeon_example:
    show bg room with dissolve
    call screen card_dungeon_screen
